@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /* ghex.h - defines GHex ;)
 
-   Copyright (C) 1998, 1999, 2000 Free Software Foundation
+   Copyright (C) 1998 - 2001 Free Software Foundation
 
    GHex is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -42,6 +42,8 @@
 
 #define MAX_MAX_UNDO_DEPTH 100000
 
+#define MESSAGE_LEN 512
+
 extern GnomeUIInfo help_menu[], file_menu[], view_menu[],
 	               main_menu[], tools_menu[];
 
@@ -95,7 +97,7 @@ typedef struct _FindDialog {
 
 typedef struct _Converter {
 	GtkWidget *window;
-	GtkWidget *entry[4];
+	GtkWidget *entry[5];
 	GtkWidget *close;
 	GtkWidget *get;
 
@@ -105,10 +107,14 @@ typedef struct _Converter {
 typedef struct {
 	GnomePrintMaster *master;
 	GnomePrintContext *pc;
+	GnomePrinter *printer;
 	GnomeFont *d_font, *h_font;
 	HexDocument *doc;
 
 	int   pages;
+	gint range;
+	gint page_first;
+	gint page_last;
 	float page_width, page_height;
 	float margin_top, margin_bottom, margin_left, margin_right;
 	float printable_width, printable_height;
@@ -122,6 +128,7 @@ typedef struct {
 	float pad_size;
 	int   offset_chars ; /* How many chars are used in the offset window */
 	int   gt;            /* group_type */
+	gboolean preview;
 } GHexPrintJobInfo;
 
 extern int restarted;
@@ -174,8 +181,9 @@ gint ask_user              (GnomeMessageBox *);
 GtkWidget *create_button   (GtkWidget *, const gchar *, gchar *);
 
 /* printing */
-void print_document        (HexDocument *doc, guint group_type,
-							GnomePrinter *printer);
+void ghex_print_job_execute(GHexPrintJobInfo *pji);
+GHexPrintJobInfo *ghex_print_job_info_new(HexDocument *doc, guint group_type);
+void ghex_print_job_info_destroy(GHexPrintJobInfo *pji);
 
 /* config stuff */
 void save_configuration    (void);
