@@ -36,7 +36,7 @@
 
 #define SCROLL_TIMEOUT 100
 
-#define is_displayable(c) (((((guchar)c)>=0x20) && (((guchar)c)<=0xFF))?1:0)
+#define is_displayable(c) ((char_widths[(guchar)c])?1:0)
 
 typedef void (*DataChangedSignal)(GtkObject *, gpointer, gpointer);
 
@@ -49,6 +49,7 @@ enum {
 static gint gtkhex_signals[LAST_SIGNAL] = { 0, 0 };
 
 GtkFixedClass *parent_class = NULL;
+gchar *char_widths = NULL;
 
 static void render_hex_lines(GtkHex *, gint, gint);
 static void render_ascii_lines(GtkHex *, gint, gint);
@@ -119,14 +120,25 @@ static void ascii_to_pointer(GtkHex *gh, gint mx, gint my) {
 
 static guint get_max_char_width(GdkFont *font) {
 	/* this is, I guess, a rather dirty trick, but
-	   right now I cant think of anything better */
+	   right now i can't think of anything better */
 	guint i;
-	guint w, maxwidth = 0;
+	guint maxwidth = 0;
+
+	if (char_widths == NULL)
+		char_widths = (gchar*)g_malloc(0x100);
+
+	char_widths[0] = 0;
+	for(i = 1; i < 0x100; i++)
+		char_widths[i] = gdk_char_width(font, (gchar)i);
 	
-	for(i = '0'; i <= 'z'; i++) {
-		w = gdk_char_width(font, (gchar)i);
-		maxwidth = MAX(maxwidth, w);
-	}
+	for(i = '0'; i <= 'z'; i++)
+		maxwidth = MAX(maxwidth, char_widths[i]);
+
+/*	for(i = 0; i < 0x100; i++)
+		if(char_widths[i] && (char_widths[i] != maxwidth)) {
+			gnome_app_warning(mdi->active_window, _("This does not appear to be a monospaced font. Ghex's display may not be perfect with this font."));
+			break;
+		}*/
 	
 	return maxwidth;
 }
@@ -240,7 +252,9 @@ static void render_byte(GtkHex *gh, gint pos) {
 	   !GTK_WIDGET_REALIZED(gh->xdisp) || !GTK_WIDGET_REALIZED(gh->adisp))
 		return;
 
-	get_xcoords(gh, pos, &cx, &cy);
+	if(!get_xcoords(gh, pos, &cx, &cy))
+		return;
+
 	format_xbyte(gh, pos, buf);
 
 	gdk_gc_set_foreground(gh->xdisp_gc, &GTK_WIDGET(gh)->style->base[GTK_STATE_NORMAL]);
@@ -253,7 +267,8 @@ static void render_byte(GtkHex *gh, gint pos) {
 					  cx, cy + gh->disp_font->ascent, buf, 2);
 	}
 	
-	get_acoords(gh, pos, &cx, &cy);
+	if(!get_acoords(gh, pos, &cx, &cy))
+		return;
 
 	gdk_gc_set_foreground(gh->adisp_gc, &GTK_WIDGET(gh)->style->base[GTK_STATE_NORMAL]);
 	gdk_draw_rectangle(gh->adisp->window, gh->adisp_gc, TRUE,
@@ -645,8 +660,6 @@ static void display_scrolled(GtkAdjustment *adj, GtkHex *gh) {
 	   (!GTK_WIDGET_DRAWABLE(gh->adisp)))
 		return;
 
-	hide_cursor(gh);
-	
 	gh->top_line = (gint)adj->value;
 	
 	rect.x = 0;
@@ -735,19 +748,18 @@ static void display_scrolled(GtkAdjustment *adj, GtkHex *gh) {
 		if(gh->offsets)
 			gtk_widget_draw(gh->offsets, &rect);
 	}
-	
-	show_cursor(gh);
 }
 
 /*
  * mouse signal handlers (button 1 and motion) for both displays
  */
-static void scroll_timeout_handler(GtkHex *gh) {
+static gboolean scroll_timeout_handler(GtkHex *gh) {
 	if(gh->scroll_dir < 0)
 		gtk_hex_set_cursor(gh, MAX(0, (int)(gh->cursor_pos - gh->cpl)));
 	else if(gh->scroll_dir > 0)
 		gtk_hex_set_cursor(gh, MIN(gh->document->file_size - 1,
 								   gh->cursor_pos + gh->cpl));    
+	return TRUE;
 }
 
 static void hex_button_cb(GtkWidget *w, GdkEventButton *event, GtkHex *gh) {
@@ -757,6 +769,7 @@ static void hex_button_cb(GtkWidget *w, GdkEventButton *event, GtkHex *gh) {
 		if(gh->scroll_timeout != -1) {
 			gtk_timeout_remove(gh->scroll_timeout);
 			gh->scroll_timeout = -1;
+			gh->scroll_dir = 0;
 		}
 		gtk_grab_remove(w);
 		gh->button = 0;
@@ -790,21 +803,26 @@ static void hex_motion_cb(GtkWidget *w, GdkEventMotion *event, GtkHex *gh) {
 		gh->scroll_dir = -1;
 	else if(y >= w->allocation.height)
 		gh->scroll_dir = 1;
-			
-	if(gh->scroll_timeout == -1)
-		gh->scroll_timeout = gtk_timeout_add(SCROLL_TIMEOUT, (GtkFunction)scroll_timeout_handler, gh);
+	else
+		gh->scroll_dir = 0;
 
+	if(gh->scroll_dir != 0) {
+		if(gh->scroll_timeout == -1)
+			gh->scroll_timeout = gtk_timeout_add(SCROLL_TIMEOUT, (GtkFunction)scroll_timeout_handler, gh);
+		return;
+	}
+	else {
+		if(gh->scroll_timeout != -1) {
+			gtk_timeout_remove(gh->scroll_timeout);
+			gh->scroll_timeout = -1;
+		}
+	}
+			
 	if(event->window != w->window)
 		return;
 
 	if((gh->active_view == VIEW_HEX) && (gh->button == 1)) {
-		if( (y <= w->allocation.height) && (y >= 0) ) {
-			if(gh->scroll_timeout != -1) {
-				gtk_timeout_remove(gh->scroll_timeout);
-				gh->scroll_timeout = -1;
-			}
-			hex_to_pointer(gh, x, y);
-		}
+		hex_to_pointer(gh, x, y);
 	}
 }
 
@@ -814,6 +832,7 @@ static void ascii_button_cb(GtkWidget *w, GdkEventButton *event, GtkHex *gh) {
 		if(gh->scroll_timeout != -1) {
 			gtk_timeout_remove(gh->scroll_timeout);
 			gh->scroll_timeout = -1;
+			gh->scroll_dir = 0;
 		}
 		gtk_grab_remove(w);
 		gh->button = 0;
@@ -845,21 +864,26 @@ static void ascii_motion_cb(GtkWidget *w, GdkEventMotion *event, GtkHex *gh) {
 		gh->scroll_dir = -1;
 	else if(y >= w->allocation.height)
 		gh->scroll_dir = 1;
-			
-	if(gh->scroll_timeout == -1)
-		gh->scroll_timeout = gtk_timeout_add(SCROLL_TIMEOUT, (GtkFunction)scroll_timeout_handler, gh);
+	else
+		gh->scroll_dir = 0;
+
+	if(gh->scroll_dir != 0) {
+		if(gh->scroll_timeout == -1)
+			gh->scroll_timeout = gtk_timeout_add(SCROLL_TIMEOUT, (GtkFunction)scroll_timeout_handler, gh);
+		return;
+	}
+	else {
+		if(gh->scroll_timeout != -1) {
+			gtk_timeout_remove(gh->scroll_timeout);
+			gh->scroll_timeout = -1;
+		}
+	}
 
 	if(event->window != w->window)
 		return;
 
 	if((gh->active_view == VIEW_ASCII) && (gh->button == 1)) {
-		if( (y <= w->allocation.height) && (y >= 0) ) {
-			if(gh->scroll_timeout != -1) {
-				gtk_timeout_remove(gh->scroll_timeout);
-				gh->scroll_timeout = -1;
-			}
-			ascii_to_pointer(gh, x, y);
-		}
+		ascii_to_pointer(gh, x, y);
 	}
 }
 
@@ -994,6 +1018,10 @@ static gint gtk_hex_key_press(GtkWidget *w, GdkEventKey *event) {
 		gtk_hex_set_cursor(gh, MIN((gint)gh->document->file_size, (gint)gh->cursor_pos + gh->vis_lines*gh->cpl));
 		break;
 	default:
+		if (event->state & GDK_MOD1_MASK) {
+			show_cursor(gh);
+			return FALSE;
+		}
 		if(gh->active_view == VIEW_HEX)
 			switch(event->keyval) {
 			case GDK_Left:
