@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /* preferences.c - setting the preferences
 
-   Copyright (C) 1998, 1999, 2000 Free Software Foundation
+   Copyright (C) 1998 - 2001 Free Software Foundation
 
    GHex is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -414,7 +414,7 @@ static void set_prefs(PropertyUI *pui) {
 	gtk_entry_set_text(GTK_ENTRY(pui->format), offset_fmt);
 	if(strcmp(offset_fmt, "%d") == 0)
 		gtk_option_menu_set_history(GTK_OPTION_MENU(pui->offset_menu), 0);
-	else if(strcmp(offset_fmt, "%x") == 0)
+	else if(strcmp(offset_fmt, "%X") == 0)
 		gtk_option_menu_set_history(GTK_OPTION_MENU(pui->offset_menu), 1);
 	else {
 		gtk_option_menu_set_history(GTK_OPTION_MENU(pui->offset_menu), 2);
